# Generated by Django 2.2.5 on 2019-09-29 21:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('spider_base', '0008_auto_20190817_1815'),
    ]

    operations = [
        migrations.AlterField(
            model_name='assignedprotection',
            name='state',
            field=models.CharField(choices=[('a', 'disabled'), ('b', 'enabled'), ('c', 'instant_fail')], default='b', help_text='State of the protection.', max_length=1),
        ),
    ]
