# Generated by Django 3.0.2 on 2020-01-04 17:35

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('spider_base', '0011_auto_20191230_1305'),
        ('spider_tags', '0007_taglayout_description'),
    ]

    operations = [
        migrations.AddField(
            model_name='spidertag',
            name='associated',
            field=models.OneToOneField(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='spider_base.AssignedContent'),
        ),
    ]
