# Generated by Django 3.0.1 on 2019-12-30 13:51

from django.db import migrations
import spkcspider.apps.spider_filets.models


class Migration(migrations.Migration):

    dependencies = [
        ('spider_filets', '0013_auto_20191230_1351'),
    ]

    operations = [
        migrations.DeleteModel(
            name='FileFilet',
        ),
        migrations.DeleteModel(
            name='TextFilet',
        ),
        migrations.CreateModel(
            name='FileFilet',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=(spkcspider.apps.spider_filets.models.LicenseMixin, 'spider_base.datacontent'),
        ),
        migrations.CreateModel(
            name='TextFilet',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=(spkcspider.apps.spider_filets.models.LicenseMixin, 'spider_base.datacontent'),
        ),
    ]
