__all__ = ["SpiderFiletsConfig"]

from django.apps import AppConfig


class SpiderFiletsConfig(AppConfig):
    name = 'spkcspider.apps.spider_filets'
    label = 'spider_filets'
    verbose_name = 'spkcspider File and Text content'
