# Generated by Django 3.0.1 on 2019-12-30 13:55

from django.db import migrations
import spkcspider.apps.spider_keys.models


class Migration(migrations.Migration):

    dependencies = [
        ('spider_keys', '0005_auto_20191230_1355'),
    ]

    operations = [
        migrations.DeleteModel(
            name='AnchorKey',
        ),
        migrations.DeleteModel(
            name='AnchorServer',
        ),
        migrations.DeleteModel(
            name='PublicKey',
        ),
        migrations.CreateModel(
            name='AnchorKey',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=(spkcspider.apps.spider_keys.models.AnchorMixing, 'spider_base.datacontent'),
        ),
        migrations.CreateModel(
            name='AnchorServer',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=(spkcspider.apps.spider_keys.models.AnchorMixing, 'spider_base.datacontent'),
        ),
        migrations.CreateModel(
            name='PublicKey',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('spider_base.datacontent',),
        ),
    ]
